@@ -121,7 +121,6 @@ static void term_handler()
 static void usage(char *program)
 {
 	printf("%s [-d] [-f restorecond_file ] [-u] [-v] \n", program);
-	exit(0);
 }
 
 void exitApp(const char *msg)
@@ -176,7 +175,7 @@ int main(int argc, char **argv)
 
 	exclude_non_seclabel_mounts();
 	atexit( done );
-	while ((opt = getopt(argc, argv, "df:uv")) > 0) {
+	while ((opt = getopt(argc, argv, "hdf:uv")) > 0) {
 		switch (opt) {
 		case 'd':
 			debug_mode = 1;
@@ -187,11 +186,16 @@ int main(int argc, char **argv)
 		case 'u':
 			run_as_user = 1;
 			break;
+		case 'h':
+			usage(argv[0]);
+			exit(0);
+			break;
 		case 'v':
 			r_opts.verbose++;
 			break;
 		case '?':
 			usage(argv[0]);
+			exit(-1);
 		}
 	}
 
